# Copyright (c) 2017 BaseSoft B.V.
# See LICENSE for details.

"""
Conversion for fixing the FME contact mess #5333
"""
import sys
import traceback

from basesoft.web.contrib.compose.models import ComposeInvitee
from basesoft.web.core.util.db import DatabaseSession
from iqportal.client.core.models import FMECompany, FMEContact
from sqlalchemy.sql.expression import and_

sa_session = DatabaseSession().Session


class FMEContactFix:
    def __init__(self, sa_session):
        self.sa_session = sa_session
        self.fixed = 0

    def get_contact(self, company):
        return self.sa_session.query(FMEContact).filter(FMEContact.company == company).first()

    def get_contacts(self, company):
        return self.sa_session.query(FMEContact).filter(FMEContact.company == company).all()

    def get_company_by_name(self, name):
        return self.sa_session.query(FMECompany).filter(FMECompany.name == name).order_by(FMECompany.id).all()

    def get_contact_by_person(self, person):
        return self.sa_session.query(FMEContact).filter(FMEContact.person == person).all()

    def get_invitee_by_contact(self, contact):
        return self.sa_session.query(ComposeInvitee).filter(ComposeInvitee.contact == contact).all()

    def get_invitee_by_contact_and_publication(self, contact, publication):
        return self.sa_session.query(ComposeInvitee).filter(and_(ComposeInvitee.contact == contact, ComposeInvitee.publication == publication)).first()

    def fix_invitees_and_remove_contacts(self, invitees, remove, target):
        print 'Updating (%s) invitees from contact %d (%s) to %d (%s)' % (len(invitees), remove.id, remove.company.name, target.id, target.company.name)
        if remove.company.name.lower() != target.company.name.lower():
            raise Exception('COMPANY NAME MISMATCH!')

        if target.deleted:
            invitees = self.get_invitee_by_contact(target)
            keep = remove
            remove = target
            target = keep
            if target.company.customer_number in (0, None):
                target.company.customer_number = remove.company.customer_number
        valid = True
        for invitee in invitees:
            t_invitee = self.get_invitee_by_contact_and_publication(target, invitee.publication)
            if t_invitee and remove.deleted:
                self.sa_session.delete(invitee)
                continue
            elif t_invitee:
                valid = False
                print 'ERROR: UNABLE TO FIX INVITEES DUE TO DUPLICATE INVITEE WITHOUT DELETED FLAG %s' % invitee.contact.company.name
                break
            invitee.contact = target
            self.sa_session.add(invitee)

        if valid:
            self.sa_session.delete(remove)
        return valid

    def convert(self):
        # get all FME companies
        total = 0
        manual = []
        for company in self.sa_session.query(FMECompany).order_by(FMECompany.name):

            # these companies have not been changed since the new customer number
            if company.last_updated.year < 2016 or (company.last_updated.year == 2016 and company.last_updated.month < 12):
                continue

            if company.company_number != company.customer_number and company.customer_number != 0:
                continue

            all_company_contacts = self.get_contacts(company)
            for contact in all_company_contacts:
                contacts = self.get_contact_by_person(contact.person)

                companies = self.get_company_by_name(company.name)
                if len(companies) == 1:
                    continue
                total += 1

                if len(contacts) == 2 and len(companies) == 2:
                    keep = contacts[0]
                    remove = contacts[1]
                    if contacts[0].company.customer_number == 0:
                        keep = contacts[1]
                        remove = contacts[0]

                    invitees = self.get_invitee_by_contact(remove)
                    if self.fix_invitees_and_remove_contacts(invitees, remove, keep):
                        self.fixed += 1
                    else:
                        manual.append(company)
                        print 'WARNING: Unable to automatically fix: "%s"' % company.name
                    continue

                if len(contacts) == 1 and len(companies) == 2:
                    self.fixed += 1
                    keep = self.get_contact(companies[0])
                    remove = self.get_contacts(companies[1])
                    for rem in remove:
                        print 'Merging company %d (%s) with %d (%s)' % (keep.company.id, keep.company.name, rem.company.id, rem.company.name)
                        if keep.company.customer_number in (0, None):
                            keep.company.customer_number = rem.company.customer_number
                        rem.company = keep.company
                        self.sa_session.add(rem)
                    self.sa_session.delete(companies[1])
                    continue

                if len(contacts) == 3 and len(companies) == 3:
                    # original company and two incorrect new ones
                    keep = contacts[0]
                    remove = []
                    if keep.created > contacts[1].created:
                        keep = contacts[1]
                        remove.append(keep)
                    else:
                        remove.append(contacts[1])
                    if keep.created > contacts[2].created:
                        keep = contacts[2]
                        remove.append(keep)
                    else:
                        remove.append(contacts[2])

                    for rem in remove:
                        invitees = self.get_invitee_by_contact(rem)
                        if not self.fix_invitees_and_remove_contacts(invitees, rem, keep):
                            manual.append(company)
                            print 'WARNING: Unable to automatically fix: "%s"' % company.name

                    self.fixed += 1
                    continue

                if len(contacts) > len(companies):
                    keep = self.get_contact(companies[0])
                    remove = self.get_contact(companies[1])
                    if keep.company.customer_number == 0:
                        keep = remove
                        remove = self.get_contact(companies[0])
                    invitees = self.get_invitee_by_contact(remove)
                    if self.fix_invitees_and_remove_contacts(invitees, remove, keep):
                        self.fixed += 1
                    else:
                        manual.append(company)
                        print 'WARNING: Unable to automatically fix: "%s"' % company.name
                    continue

                if len(contacts) == 2 and len(companies) == 3:
                    # Original contact had a different person
                    keep = None
                    for c in companies:
                        x = self.get_contact(c)
                        if x not in contacts:
                            continue
                        if x.company.customer_number == 0:
                            remove = x
                            continue
                        if keep is None:
                            keep = x
                            continue
                        remove = x
                    if keep is None:
                        manual.append(company)
                        print 'WARNING: Unable to automatically fix: "%s"' % company.name
                        continue
                    invitees = self.get_invitee_by_contact(remove)
                    if self.fix_invitees_and_remove_contacts(invitees, remove, keep):
                        self.fixed += 1
                    else:
                        manual.append(company)
                        print 'WARNING: Unable to automatically fix: "%s"' % company.name
                    continue

                manual.append(company)
                print 'WARNING: Unable to automatically fix: "%s"' % companies[0].name

        print 'TOTAL: ', total, ' FIXED: ', self.fixed
        print 'Manually repair: %s' % ', '.join(['%d (%s)' % (c.id, c.name) for c in set(manual)])


DRYRUN = True

if __name__ == "__main__":
    sa_session.begin()
    try:
        f = FMEContactFix(sa_session)

        f.convert()

        if DRYRUN:
            sa_session.rollback()
        else:
            sa_session.commit()
            print 'DONE, this script cant be ran another time.'
    except Exception, e:

        traceback.print_exc(file=sys.stdout)

        sa_session.rollback()
        raise e


