# coding=utf-8
# Copyright (c) 2017 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.core.management.commands.iqubesync import ProjectBase, CubeBase
from iqportal.datasources.cbs.controllers import CBSController
from iqportal.iqube.core.models import DataSource


# create entry in tblDataSource
class CBSProject(ProjectBase):
    # prefix + project_name for the mondrian_project name in the tblDataSource
    project_name = 'CBSProject'
    description = '(import) CBS data'

    def __init__(self, sa_session, controller):
        ProjectBase.__init__(self, sa_session, controller)
        self._db_from_connstr(settings.DB_CONNSTR_CBS)


    def get_cubes(self):
        """
       creates entries in the tblDataSourceCube for every table in the cbs database.

       :return:
       """
        self.cbs_controller = CBSController(self.sa_session)

        # get all cbs tables from cbs database (list), e.g. [u'80728ned',u'82825NED']
        tables = self.cbs_controller.get_tables()

        # create new list called returnvalue (rv)
        rv = []

        # loop tables, create CubeBase instance with xml property
        for table in tables:

            # create a cubebase instance
            cb = CubeBase(self.sa_session)

            # assign a name property to the cubebase, equal to table_name
            cb.name = table

            # assign xml property to new cubebase
            cb.xml = self.cbs_controller.get_cube_xml(table)

            # add cubebase to list of returnvalues
            rv.append(cb)

        return rv


projects = (CBSProject,)
