from django.utils.translation import ugettext as _

from basesoft.web.core.exceptions import PublicUserException
from iqportal.datasources.cbs.controllers import CBSController
from iqportal.iqube.core.controllers import ItemResultController


class CBSHandler:

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.cbs = CBSController(self.sa_session, account_id=self.account_id)

    def get_all_datasources(self):
        """
        returns a list of datasources and identifiers
        :return:
        """

        all_datasources = self.cbs.get_all_tables()

        #returns a list of all non-imported datasources, sorted alphabetically
        return sorted(all_datasources, key=lambda k: k['label'].lower())

    def import_data(self, cbs_id):
        """

        - return True added, otherwise no return statement
        - import_data only writes to db, doesn't return anything

        :param cbs_id:
        :return:
        """

        self.cbs.import_data(cbs_id)
        return True

    def get_cube_names(self):
        """
        returns a list of cubenames for imported datasources
        :return:
        """

        table_names = self.cbs.get_all_imported_cubes()
        return sorted(table_names, key=lambda  k:k['label'].lower())

    def make_configurable_widget_config(self, cube_name):
        """
        make new configurable widget for cbs

        :param cube_name:
        :return:
        """
        if cube_name in (None, "", ):
            raise PublicUserException(_("Please select a cube."))

        # extract datasources for CBSProject
        datasource_id = self.cbs.get_datasource().id
        c = ItemResultController(self.sa_session, account_id=self.account_id)
        return c.make_configurable_widget_config(datasource_id, cube_name)

handlers = (CBSHandler, )

