# coding=utf-8
# Copyright (c) 2017 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.core.management.commands.iqubesync import ProjectBase, CubeBase
from basesoft.web.core.util.controllers import lazy_assert
from iqportal.datasources.oxford.controllers import OxfordController
from iqportal.iqube.core.models import DataSource


class OxfordCube(CubeBase):
    def __init__(self, databank_id, databank_name, *args, **kwargs):
        super(OxfordCube, self).__init__(*args, **kwargs)

        lazy_assert(databank_id, int)
        lazy_assert(databank_name, str)
        self.databank_id = databank_id
        self.databank_name = databank_name

        self.name = 'Oxford%s' % databank_name

    def _xml(self):
        return """
    <Cube name="Oxford%(databank_name)s" defaultMeasure="Value" >

        <Table name="oxford_fact" alias="fact_%(databank_id)s" >
            <SQL dialect="mysql">
                fact_%(databank_id)s.databank_id = %(databank_id)s
            </SQL>
        </Table>

        <Dimension name="Location" foreignKey="location_id">
            <Hierarchy hasAll="true" primaryKey="id" primaryKeyTable="oxford_location">
                <Table name="oxford_location"/>
                <Level name="Location" table="oxford_location" column="id" type="Integer" captionColumn="title" ordinalColumn="title" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Indicator" foreignKey="indicator_id" >
            <Hierarchy hasAll="true" primaryKey="id" primaryKeyTable="oxford_indicator">
                <Table name="oxford_indicator" alias="indicator_%(databank_id)s">
                    <SQL dialect="mysql">
                        indicator_%(databank_id)s.databank_id = %(databank_id)s
                    </SQL>
                </Table>
                <Level name="Indicator" table="indicator_%(databank_id)s" column="id" type="Integer" captionColumn="name" ordinalColumn="display_order" />
            </Hierarchy>
        </Dimension>
        

        <Dimension name="Year">
            <Hierarchy hasAll="true">
                <Level name="Year" type="Integer" column="year" ordinalColumn="year" />
            </Hierarchy>
        </Dimension>
        
        <Dimension name="Period">
            <Hierarchy hasAll="true">
                <Level name="Period" column="period" type="Integer">
                    <CaptionExpression>
                        <SQL dialect="mysql">
                            CASE `period`
                                WHEN 1 THEN 'Q1'
                                WHEN 2 THEN 'Q2'
                                WHEN 3 THEN 'Q3'
                                WHEN 4 THEN 'Q4'
                                WHEN 5 THEN 'Annual'
                                ELSE 'Undefined' END
                        </SQL>
                    </CaptionExpression>
                </Level>
            </Hierarchy>
        </Dimension>

        <Measure name="Value" column="value" aggregator="sum" datatype="Numeric" formatString="#,##0.00" />

    </Cube>
        """ % {
            'databank_id': self.databank_id,
            'databank_name': self.databank_name,
        }

    xml = property(_xml)

# create entry in tblDataSource
class OxfordProject(ProjectBase):
    project_name = 'OxfordProject'

    def __init__(self, sa_session, controller):
        ProjectBase.__init__(self, sa_session, controller)
        self._db_from_connstr(settings.DB_CONNSTR_OXFORD)

    def get_cubes(self):
        controller = OxfordController(self.sa_session)

        rv = []

        for row in controller.get_imported_databanks():
            cb = OxfordCube(row['id'], row['code'], self.sa_session)
            rv.append(cb)

        return rv


projects = (OxfordProject,)
